import contextvars
import inspect
from contextlib import contextmanager
from typing import List


class Supplier:
    def __init__(self, name):
        self.contextvar = contextvars.ContextVar(name)

    @contextmanager
    def use(self, value):
        token = self.contextvar.set(value)
        try:
            yield None
        finally:
            self.contextvar.reset(token)

    def get(self):
        return self.contextvar.get()


def supply(*suppliers: List[Supplier]):
    def supply_inner(fn):
        # No good way to detect whether the decorated method is an instance method
        # Instead we just check to see if the first argument has name "self"
        params = inspect.signature(fn).parameters
        should_keep_first_arg_in_place = len(params) and next(iter(params)) == "self"

        def wrapped(*args, **kwargs):
            prepended = tuple(supplier.get() for supplier in suppliers)
            if not should_keep_first_arg_in_place:
                new_args = prepended + args
            else:
                new_args = args[:1] + prepended + args[1:]
            return fn(*new_args, **kwargs)

        return wrapped

    return supply_inner




from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()


setup(
    name="supplier",
    version="0.0.1",
    author="Evin Sellin",
    author_email="evinism@gmail.com",
    description="Simple library for passing values deeply",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/evinism/supplier",
    packages=find_packages(),
    install_requires=['contextvars;python_version<"3.7"'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)